<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 115 $
 * $Id: ClientContactListFactory.class.php 115 2006-09-15 23:48:31Z ipso $
 * $Date: 2006-09-15 16:48:31 -0700 (Fri, 15 Sep 2006) $
 */
class ClientContactListFactory extends ClientContactFactory implements IteratorAggregate {

	function getAll($limit = NULL, $page = NULL, $where = NULL, $order = NULL) {
		$query = '
					select 	*
					from	'. $this->getTable() .'
					WHERE deleted = 0';
		$query .= $this->getWhereSQL( $where );
		$query .= $this->getSortSQL( $order );

		if ($limit == NULL) {
			//Run query without limit
			$this->rs = $this->db->SelectLimit($query);
		} else {
			$this->rs = $this->db->PageExecute($query, $limit, $page);
		}

		return $this;
	}

	function getById($id, $where = NULL, $order = NULL) {
		if ( $id == '' ) {
			return FALSE;
		}

		$this->rs = $this->getCache($id);
		if ( $this->rs === FALSE ) {

			$ph = array(
						'id' => $id,
						);

			$query = '
						select 	*
						from	'. $this->getTable() .'
						where	id = ?
							AND deleted = 0';
			$query .= $this->getWhereSQL( $where );
			$query .= $this->getSortSQL( $order );

			$this->rs = $this->db->Execute($query, $ph);

			$this->saveCache($this->rs,$id);
		}

		return $this;
	}

	function getByIdAndClientId($id, $client_id, $where = NULL, $order = NULL) {
		if ( $id == '' ) {
			return FALSE;
		}

		if ( $client_id == '' ) {
			return FALSE;
		}

		$ph = array(
					'id' => $id,
					'client_id' => $client_id,
					);

		$query = '
					select 	*
					from	'. $this->getTable() .'
					where	id = ?
						AND client_id = ?
						AND deleted = 0';
		$query .= $this->getWhereSQL( $where );
		$query .= $this->getSortSQL( $order );

		$this->rs = $this->db->Execute($query, $ph);

		return $this;
	}

	function getByIdAndCompanyId($id, $company_id, $where = NULL, $order = NULL) {
		if ( $id == '' ) {
			return FALSE;
		}

		if ( $company_id == '' ) {
			return FALSE;
		}

		$cf = new ClientFactory();

		$ph = array(
					'id' => $id,
					'company_id' => $company_id,
					);

		$query = '
					select 	a.*
					from	'. $this->getTable() .' as a,
							'. $cf->getTable() .' as b
					where 	a.client_id = b.id
						AND a.id = ?
						AND b.company_id = ?
						AND ( a.deleted = 0 AND b.deleted=0 )
					';
		$query .= $this->getWhereSQL( $where );
		$query .= $this->getSortSQL( $order );

		$this->rs = $this->db->Execute($query, $ph);

		return $this;
	}

	function getByClientIdAndCompanyId($client_id, $company_id, $where = NULL, $order = NULL) {
		if ( $client_id == '' ) {
			return FALSE;
		}

		if ( $company_id == '' ) {
			return FALSE;
		}

		$cf = new ClientFactory();

		$ph = array(
					'client_id' => $client_id,
					'company_id' => $company_id,
					);

		$query = '
					select 	a.*
					from	'. $this->getTable() .' as a,
							'. $cf->getTable() .' as b
					where 	a.client_id = b.id
						AND a.client_id = ?
						AND b.company_id = ?
						AND ( a.deleted = 0 AND b.deleted=0 )
					';
		$query .= $this->getWhereSQL( $where );
		$query .= $this->getSortSQL( $order );

		$this->rs = $this->db->Execute($query, $ph);

		return $this;
	}

	function getByTypeIdAndDefaultAndClientIdAndCompanyId($type_id, $default, $client_id, $company_id, $where = NULL, $order = NULL) {
		if ( $type_id == '' ) {
			return FALSE;
		}

		if ( $default == '' ) {
			return FALSE;
		}

		if ( $client_id == '' ) {
			return FALSE;
		}

		if ( $company_id == '' ) {
			return FALSE;
		}

		$cf = new ClientFactory();

		$ph = array(
					'type_id' => $type_id,
					'default' => $this->toBool($default),
					'client_id' => $client_id,
					'company_id' => $company_id,
					);

		//AND a.is_default = '. $this->db->qstr( $this->toBool($default) ) .'
		$query = '
					select 	a.*
					from	'. $this->getTable() .' as a,
							'. $cf->getTable() .' as b
					where 	a.client_id = b.id
						AND a.type_id = ?
						AND a.is_default = ?
						AND a.client_id = ?
						AND b.company_id = ?
						AND ( a.deleted = 0 AND b.deleted=0 )
					';
		$query .= $this->getWhereSQL( $where );
		$query .= $this->getSortSQL( $order );

		$this->rs = $this->db->Execute($query, $ph);

		return $this;
	}

	function getByClientIdAndCompanyIdArray($client_id, $company_id, $include_blank = TRUE ) {
		$cclf = new ClientContactListFactory();
		$cclf->getByClientIdAndCompanyId( $client_id, $company_id );

		if ( $include_blank == TRUE ) {
			$list[0] = '--';
		}

		foreach ($cclf as $cc_obj) {
			$list[$cc_obj->getID()] = $cc_obj->getFullName();
		}

		return $list;
	}

	function getAllArray() {
		$clf = new ClientListFactory();
		$clf->getAll();

		$list[0] = '--';

		foreach ($clf as $c_obj) {
			$list[$c_obj->getID()] = $c_obj->getCompanyName();
		}

		return $list;
	}
}
?>
