<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 413 $
 * $Id: HierarchyControlFactory.class.php 413 2006-12-05 00:33:22Z ipso $
 * $Date: 2006-12-04 16:33:22 -0800 (Mon, 04 Dec 2006) $
 */
class HierarchyControlFactory extends Factory {
	protected $table = 'hierarchy_control';
	protected $pk_sequence_name = 'hierarchy_control_id_seq'; //PK Sequence name

	//Temporaray holding array.
	protected $object_type_ids = NULL;

	protected $company_validator_msg = 'Invalid Company';
	protected $company_validator_label = 'company';

	protected $name_validator_unique_msg = 'Name is already in use';
	protected $name_validator_islength_msg = 'Name is invalid';
	protected $name_validator_label = 'name';

	protected $description_validator_islength_msg = 'Description is invalid';
	protected $description_validator_label = 'description';

	protected $objecttype_validator_msg = 'Incorrect Object type selection';
	protected $objecttype_validator_unique_msg = 'Object type is already assigned to another hierarchy';
	protected $objecttype_validator_min_msg = 'At least one object type must be selected';
	protected $objecttype_validator_label = 'object_type';

	function getCompany() {
		return $this->data['company_id'];
	}
	function setCompany($id) {
		$id = trim($id);

		$clf = new CompanyListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->company_validator_label,
													$clf->getByID($id),
													$this->company_validator_msg
													) ) {

			$this->data['company_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function isUniqueName($name) {
		$ph = array(
					'company_id' => $this->getCompany(),
					'name' => $name,
					);

		$query = 'select id from '. $this->getTable() .' where company_id = ? AND name = ? AND deleted=0';
		$default_schedule_control_id = $this->db->GetOne($query, $ph);
		Debug::Arr($default_schedule_control_id,'Unique Hierarchy Control ID: '. $default_schedule_control_id, __FILE__, __LINE__, __METHOD__,10);

		if ( $default_schedule_control_id === FALSE ) {
			return TRUE;
		} else {
			if ($default_schedule_control_id == $this->getId() ) {
				return TRUE;
			}
		}

		return FALSE;
	}
	function getName() {
		return $this->data['name'];
	}
	function setName($name) {
		$name = trim($name);

		if (	$this->Validator->isLength(	$this->name_validator_label,
											$name,
											$this->name_validator_islength_msg,
											2,50)
				AND	$this->Validator->isTrue(	$this->name_validator_label,
												$this->isUniqueName($name),
												$this->name_validator_unique_msg
												)
						) {

			$this->data['name'] = $name;

			return TRUE;
		}

		return FALSE;
	}

	function getDescription() {
		return $this->data['description'];
	}
	function setDescription($description) {
		$description = trim($description);

		if (	$description == ''
				OR $this->Validator->isLength(	$this->description_validator_label,
											$description,
											$this->description_validator_islength_msg,
											1,255) ) {

			$this->data['description'] = $description;

			return TRUE;
		}

		return FALSE;
	}

	//Return the temp IDs for validation.
	function getTmpObjectType() {
		if ( isset( $this->object_type_ids ) ) {
			return 	$this->object_type_ids;
		}

		return FALSE;
	}


	//Return IDs from the database,
	function getObjectType() {
		$hotlf = new HierarchyObjectTypeListFactory();
		$hotlf->getByHierarchyControlId( $this->getId() );

		foreach ($hotlf as $object_type) {
			$object_type_list[] = $object_type->getObjectType();
		}

		if ( isset($object_type_list) ) {
			return $object_type_list;
		}

		return FALSE;
	}

	function setObjectType($ids) {
		if ( is_array($ids) AND count($ids) > 0 ) {
			$this->object_type_ids = $ids;

			return TRUE;
		}

		$this->Validator->isTrue(		$this->objecttype_validator_label,
										FALSE,
										$this->objecttype_validator_min_msg);
		return FALSE;
	}

	function setObjectTypeIds($ids) {
		if (is_array($ids) and count($ids) > 0) {

			//If needed, delete mappings first.
			$hotlf = new HierarchyObjectTypeListFactory();
			$hotlf->getByHierarchyControlId( $this->getId() );

			$object_type_ids = array();
			foreach ($hotlf as $object_type) {
				$object_type_id = $object_type->getId();
				Debug::text('Object Type ID: '. $object_type_id, __FILE__, __LINE__, __METHOD__, 10);

				//Delete object types that are not selected.
				if ( !in_array($object_type_id, $ids) ) {
					Debug::text('Deleting ObjectType: '. $object_type_id, __FILE__, __LINE__, __METHOD__, 10);
					$object_type->Delete();
				} else {
					//Save object type ID's that need to be updated.
					Debug::text('NOT Deleting object type: '. $object_type_id, __FILE__, __LINE__, __METHOD__, 10);
					$object_type_ids[] = $object_type_id;
				}
			}

			//Insert new mappings.
			$hotf = new HierarchyObjectTypeFactory();
			foreach ($ids as $id) {
				if ( !in_array($id, $object_type_ids) ) {
					$hotf->setHierarchyControl( $this->getId() );
					$hotf->setObjectType( $id );

					if ($this->Validator->isTrue(		$this->objecttype_validator_label,
														$hotf->Validator->isValid(),
														$this->objecttype_validator_unique_msg)) {
						$hotf->save();
					}
				}
			}

			return TRUE;
		}

		return FALSE;
	}

	function Validate() {
		$hotlf = new HierarchyObjectTypeListFactory();
		$hotlf->getByCompanyId( $this->getCompany() );
		foreach ( $hotlf as $object_type_obj) {
			if ( $object_type_obj->getHierarchyControl() != $this->getId() ) {
				$current_object_type_ids[] = $object_type_obj->getObjectType();
			}
		}

		if ( ( is_array( $this->getTmpObjectType() ) AND count( $this->getTmpObjectType() ) > 0 )
				AND ( isset($current_object_type_ids) AND count($current_object_type_ids) > 0 ) ) {
			Debug::text('Other object types are assigned, checking for duplicates: '. count( $this->getTmpObjectType() ), __FILE__, __LINE__, __METHOD__, 10);

            //Debug::Arr($current_object_type_ids, 'Already assigned Ids: ', __FILE__, __LINE__, __METHOD__, 10);
			//Debug::Arr($this->getTmpObjectType(), 'Tmp Ids: ', __FILE__, __LINE__, __METHOD__, 10);

			foreach( $current_object_type_ids as $current_object_type_id) {
				if ( in_array( $current_object_type_id, $this->getTmpObjectType() ) ) {
					Debug::text('Duplicate object type IDs found!: ', __FILE__, __LINE__, __METHOD__, 10);
					$this->Validator->isTrue(		$this->objecttype_validator_label,
													FALSE,
													$this->objecttype_validator_unique_msg);

					break;
				}

			}
		}

		return TRUE;
	}

	function postSave() {
		//Save objectype IDs

		$this->setObjectTypeIds( $this->getTmpObjectType() );

		return TRUE;

	}
}
?>
