<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 589 $
 * $Id: ExceptionPolicyFactory.class.php 589 2007-01-30 22:44:08Z ipso $
 * $Date: 2007-01-30 14:44:08 -0800 (Tue, 30 Jan 2007) $
 */
class ExceptionPolicyFactory extends Factory {
	protected $table = 'exception_policy';
	protected $pk_sequence_name = 'exception_policy_id_seq'; //PK Sequence name

	protected $type_options = array(
										//Schedule Exceptions
										'A' => 'Unscheduled Absence',
										'B' => 'Not Scheduled',
										'C' => 'In Early',
										'D' => 'In Late',
										'E' => 'Out Early',
										'F' => 'Out Late',
										'G' => 'Over Scheduled Hours',
										'H' => 'Under Scheduled Hours',
//										'I' => 'Branch Mismatch',
//										'J' => 'Department Mismatch',

										//Punch Exceptions
										'K' => 'Missing In Punch',
										'L' => 'Missing Out Punch',
//										'M' => 'Long Lunch',
//										'N' => 'Short Lunch',
										'O' => 'No Lunch',
										'P' => 'Missing Lunch In/Out Punch',
//										'P' => 'Long Break',
//										'Q' => 'Too Many Breaks'

										//Job Exceptions
/*
										'R' => 'Job Mismatch',
										'S' => 'Task Mismatch',
										'T' => 'Not Allowed On Job',
										'U' => 'Not Allowed On Task',
										'V' => 'Job Completed',
*/
									);

	protected $enable_grace = array('C', 'D', 'E','F','M','N');
	protected $enable_watch_window = array('C', 'D', 'E','F');

	protected $severity_options = array(
											10 => 'Low',
											20 => 'Medium',
											30 => 'High'
								);

	protected $type_validator_options_msg = 'Incorrect Type';
	protected $type_validator_label = 'type';

	protected $severity_validator_options_msg = 'Incorrect Severity';
	protected $severity_validator_label = 'severity';

	protected $exceptionpolicycontrol_validator_msg = 'Exception Policy Control is invalid';
	protected $exceptionpolicycontrol_validator_label = 'exception_policy_control';

	protected $watchwindow_validator_isnumeric_msg = 'Incorrect Watch Window';
	protected $watchwindow_validator_label = 'watch_window';

	protected $grace_validator_isnumeric_msg = 'Incorrect grace value';
	protected $grace_validator_label = 'grace';

	protected $demerit_validator_isnumeric_msg = 'Incorrect demerit value';
	protected $demerit_validator_label = 'demerit';

	function getExceptionPolicyControl() {
		if ( isset($this->data['exception_policy_control_id']) ) {
			return $this->data['exception_policy_control_id'];
		}

		return FALSE;
	}
	function setExceptionPolicyControl($id) {
		$id = trim($id);

		$epclf = new ExceptionPolicyControlListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->exceptionpolicycontrol_validator_label,
													$epclf->getByID($id),
													$this->exceptionpolicycontrol_validator_msg
													) ) {

			$this->data['exception_policy_control_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getType() {
		if ( isset($this->data['type_id']) ) {
			return $this->data['type_id'];
		}

		return FALSE;
	}
	function setType($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('type') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->type_validator_label,
											$value,
											$this->type_validator_options_msg,
											$this->getOptions('type')) ) {

			$this->data['type_id'] = $value;

			return FALSE;
		}

		return FALSE;
	}

	function getSeverity() {
		if ( isset($this->data['severity_id']) ) {
			return $this->data['severity_id'];
		}

		return FALSE;
	}
	function setSeverity($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('severity') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->severity_validator_label,
											$value,
											$this->severity_validator_options_msg,
											$this->getOptions('severity')) ) {

			$this->data['severity_id'] = $value;

			return FALSE;
		}

		return FALSE;
	}

	function getWatchWindow() {
		if ( isset($this->data['watch_window']) ) {
			return $this->data['watch_window'];
		}

		return FALSE;
	}
	function setWatchWindow($value) {
		$value = trim($value);

		if 	(	$this->Validator->isNumeric(		$this->watchwindow_validator_label,
													$value,
													$this->watchwindow_validator_isnumeric_msg) ) {

			$this->data['watch_window'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getGrace() {
		if ( isset($this->data['grace']) ) {
			return $this->data['grace'];
		}

		return FALSE;
	}
	function setGrace($value) {
		$value = trim($value);

		if 	(	$this->Validator->isNumeric(		$this->grace_validator_label,
													$value,
													$this->grace_validator_isnumeric_msg) ) {

			$this->data['grace'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getDemerit() {
		if ( isset($this->data['demerit']) ) {
			return $this->data['demerit'];
		}

		return FALSE;
	}
	function setDemerit($value) {
		$value = trim($value);

		if 	(	$this->Validator->isNumeric(		$this->demerit_validator_label,
													$value,
													$this->demerit_validator_isnumeric_msg) ) {

			$this->data['demerit'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getActive() {
		return $this->fromBool( $this->data['active'] );
	}
	function setActive($bool) {
		$this->data['active'] = $this->toBool($bool);

		return true;
	}

	function isEnabledGrace( $code ) {
		if ( in_array( $code, $this->enable_grace ) ) {
			return TRUE;
		}

		return FALSE;
	}

	function isEnabledWatchWindow( $code ) {
		if ( in_array( $code, $this->enable_watch_window ) ) {
			return TRUE;
		}

		return FALSE;
	}

	function calcExceptions ( $user_date_id, $enable_future_exceptions = TRUE ) {
		global $profiler;

		$profiler->startTimer( "ExceptionPolicy::calcExceptions()");

		if ( $user_date_id == '' ) {
			return FALSE;
		}
		Debug::text(' User Date ID: '. $user_date_id , __FILE__, __LINE__, __METHOD__,10);

		//Get user date info
		$udlf = new UserDateListFactory();
		$udlf->getById( $user_date_id );
		if ( $udlf->getRecordCount() > 0 ) {
			$user_date_obj = $udlf->getCurrent();
			
			if ( $enable_future_exceptions == FALSE
					AND $user_date_obj->getDateStamp() > TTDate::getEndDayEpoch() ) {
				return FALSE;
			}
		} else {
			return FALSE;
		}

		//Since we are not usng demerits yet, just always delete exceptions and re-calculate them
		$elf = new ExceptionListFactory();
		$elf->getByUserDateID( $user_date_id );
		if ( $elf->getRecordCount() > 0 ) {
			foreach( $elf as $e_obj ) {
				Debug::text(' Deleting Exception: '.  $e_obj->getID(), __FILE__, __LINE__, __METHOD__,10);
				$e_obj->Delete();
			}
		}


		//Get all Punches on this date for this user.
		$plf = new PunchListFactory();
		$plf->getByUserDateId( $user_date_id );
		if ( $plf->getRecordCount() > 0 ) {
			Debug::text(' Found Punches: '.  $plf->getRecordCount(), __FILE__, __LINE__, __METHOD__,10);
		}

		$slf = new ScheduleListFactory();
		$slf->getByUserDateIdAndStatusId( $user_date_id, 10 );
		if ( $slf->getRecordCount() > 0 ) {
			Debug::text(' Found Schedule: '.  $slf->getRecordCount(), __FILE__, __LINE__, __METHOD__,10);
		}

		//Get all active exceptions.
		$eplf = new ExceptionPolicyListFactory();
		$eplf->getByPolicyGroupUserIdAndActive( $user_date_obj->getUser(), TRUE );
		if ( $eplf->getRecordCount() > 0 ) {
			Debug::text(' Found Active Exceptions: '.  $eplf->getRecordCount(), __FILE__, __LINE__, __METHOD__,10);

			foreach ( $eplf as $ep_obj )  {
				Debug::text(' Found Exception Type: '. $ep_obj->getType() , __FILE__, __LINE__, __METHOD__,10);

				switch ( strtolower( $ep_obj->getType() ) ) {
					case 'a': //Unschedule Absence... Anytime they are scheduled and have not punched in.
						if( $plf->getRecordCount() == 0 ) {

							if ( $slf->getRecordCount() > 0 ) {
								foreach( $slf as $s_obj ) {
									if ( $s_obj->getStatus() == 10 ) {
										$ef = new ExceptionFactory();
										$ef->setUserDateID( $user_date_id );
										$ef->setExceptionPolicyID( $ep_obj->getId() );
										$ef->setType( 10 );
										$ef->setEnableDemerits( TRUE );
										if ( $ef->isValid() ) {
											$ef->Save();
										}
									}
								}
							} else {
								Debug::text(' NOT Scheduled', __FILE__, __LINE__, __METHOD__,10);
							}
						}

						break;
					case 'c': //In Early
						//Loop through each punch, find out if they are scheduled, and if they are in early
						foreach ( $plf as $p_obj ) {
							if ( $p_obj->getType() == 10 AND $p_obj->getStatus() == 10 ) { //Normal In
								if ( $p_obj->setScheduleID() == TRUE ) {
									if ( $p_obj->getTimeStamp() < $p_obj->getScheduleObject()->getStartTime() ) {
										if ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getStartTime(), $ep_obj->getGrace() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;Within Grace time, IGNORE EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);
										} elseif ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getStartTime(), $ep_obj->getWatchWindow() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NOT Within Grace time, SET EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);

											$ef = new ExceptionFactory();
											$ef->setUserDateID( $user_date_id );
											$ef->setExceptionPolicyID( $ep_obj->getId() );
											$ef->setPunchID( $p_obj->getID() );
											$ef->setType( 10 );
											$ef->setEnableDemerits( TRUE );
											if ( $ef->isValid() ) {
												$ef->Save();
											}
										}

									}
								} else {
									Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NO Schedule Found', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						}

						break;
					case 'd': //In Late
						foreach ( $plf as $p_obj ) {
							if ( $p_obj->getType() == 10 AND $p_obj->getStatus() == 10 ) { //Normal In
								if ( $p_obj->setScheduleID() == TRUE ) {
									if ( $p_obj->getTimeStamp() > $p_obj->getScheduleObject()->getStartTime() ) {
										if ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getStartTime(), $ep_obj->getGrace() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;Within Grace time, IGNORE EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);
										} elseif (  TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getStartTime(), $ep_obj->getWatchWindow() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NOT Within Grace time, SET EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);

											$ef = new ExceptionFactory();
											$ef->setUserDateID( $user_date_id );
											$ef->setExceptionPolicyID( $ep_obj->getId() );
											$ef->setPunchID( $p_obj->getID() );
											$ef->setType( 10 );
											$ef->setEnableDemerits( TRUE );
											if ( $ef->isValid() ) {
												$ef->Save();
											}
										}

									}
								} else {
									Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NO Schedule Found', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						}

						break;
					case 'e': //Out Early
						//Loop through each punch, find out if they are scheduled, and if they are in early
						foreach ( $plf as $p_obj ) {
							if ( $p_obj->getType() == 10 AND $p_obj->getStatus() == 20 ) { //Normal Out
								if ( $p_obj->setScheduleID() == TRUE ) {
									if ( $p_obj->getTimeStamp() < $p_obj->getScheduleObject()->getEndTime() ) {
										if ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getEndTime(), $ep_obj->getGrace() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;Within Grace time, IGNORE EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);
										} elseif ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getEndTime(), $ep_obj->getWatchWindow() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NOT Within Grace time, SET EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);

											$ef = new ExceptionFactory();
											$ef->setUserDateID( $user_date_id );
											$ef->setExceptionPolicyID( $ep_obj->getId() );
											$ef->setPunchID( $p_obj->getID() );
											$ef->setType( 10 );
											$ef->setEnableDemerits( TRUE );
											if ( $ef->isValid() ) {
												$ef->Save();
											}
										}

									}
								} else {
									Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NO Schedule Found', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						}

/*
								$p_obj->inScheduleStartStopWindow( $p_obj->getTimeStamp() );
								Debug::text('&nbsp;&nbsp;Schedule Time: '. TTDate::getDate('DATE+TIME', $p_obj->getScheduleWindowTime() ) .' Current Punch Time: '. TTDate::getDate('DATE+TIME', $p_obj->getTimeStamp() ) .' Grace Time: '. $ep_obj->getGrace() .' Watch Window: '. $ep_obj->getWatchWindow() , __FILE__, __LINE__, __METHOD__,10);

								if ( $p_obj->getTimeStamp() < $p_obj->getScheduleWindowTime()
										OR TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleWindowTime(), $ep_obj->getWatchWindow() ) == TRUE ) {
									Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;Within Watch Window: ', __FILE__, __LINE__, __METHOD__,10);

									if ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleWindowTime(), $ep_obj->getGrace() ) == TRUE ) {
										Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;Within Grace time, IGNORE EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);
									} else {
										Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NOT Within Grace time, SET EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);

										$ef = new ExceptionFactory();
										$ef->setUserDateID( $user_date_id );
										$ef->setExceptionPolicyID( $ep_obj->getId() );
										$ef->setPunchID( $p_obj->getID() );
										$ef->setType( 10 );
										$ef->setEnableDemerits( TRUE );
										if ( $ef->isValid() ) {
											$ef->Save();
										}
									}
								} else {
									Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NOT Within Watch Window: ', __FILE__, __LINE__, __METHOD__,10);
								}
*/

						break;
					case 'f': //Out Late
						foreach ( $plf as $p_obj ) {
							if ( $p_obj->getType() == 10 AND $p_obj->getStatus() == 20 ) { //Normal Out
								if ( $p_obj->setScheduleID() == TRUE ) {
									if ( $p_obj->getTimeStamp() > $p_obj->getScheduleObject()->getEndTime() ) {
										if ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getEndTime(), $ep_obj->getGrace() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;Within Grace time, IGNORE EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);
										} elseif ( TTDate::inWindow( $p_obj->getTimeStamp(), $p_obj->getScheduleObject()->getEndTime(), $ep_obj->getWatchWindow() ) == TRUE ) {
											Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NOT Within Grace time, SET EXCEPTION: ', __FILE__, __LINE__, __METHOD__,10);

											$ef = new ExceptionFactory();
											$ef->setUserDateID( $user_date_id );
											$ef->setExceptionPolicyID( $ep_obj->getId() );
											$ef->setPunchID( $p_obj->getID() );
											$ef->setType( 10 );
											$ef->setEnableDemerits( TRUE );
											if ( $ef->isValid() ) {
												$ef->Save();
											}
										}

									}
								} else {
									Debug::text('&nbsp;&nbsp;&nbsp;&nbsp;NO Schedule Found', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						}

						break;
					case 'k': //Missing In Punch
						foreach ( $plf as $p_obj ) {
							Debug::text(' Punch: Status: '. $p_obj->getStatus() .' Punch Control ID: '. $p_obj->getPunchControlID() .' TimeStamp: '. $p_obj->getTimeStamp(), __FILE__, __LINE__, __METHOD__,10);
							$punch_pairs[$p_obj->getPunchControlID()][] = $p_obj->getStatus();
						}

						if ( isset($punch_pairs) ) {
							foreach($punch_pairs as $punch_control_id => $punch_pair) {
								Debug::Arr($punch_pair, 'Punch Pair for Control ID:'. $punch_control_id, __FILE__, __LINE__, __METHOD__,10);

								if ( count($punch_pair) != 2 ) {
									Debug::text('Found Missing Punch: ', __FILE__, __LINE__, __METHOD__,10);

									if ( $punch_pair[0] == 20 ) { //Missing In Punch
										Debug::text('Found Missing In Punch: ', __FILE__, __LINE__, __METHOD__,10);

										$ef = new ExceptionFactory();
										$ef->setUserDateID( $user_date_id );
										$ef->setExceptionPolicyID( $ep_obj->getId() );
										$ef->setPunchControlID( $p_obj->getPunchControlID() );
										$ef->setType( 10 );
										$ef->setEnableDemerits( TRUE );
										if ( $ef->isValid() ) {
											$ef->Save();
										}

									}
								} else {
									Debug::text('No Missing Punches...', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						}
						unset($punch_pairs, $punch_pair);

						break;
					case 'l': //Missing Out Punch
						foreach ( $plf as $p_obj ) {
							Debug::text(' Punch: Status: '. $p_obj->getStatus() .' Punch Control ID: '. $p_obj->getPunchControlID() .' TimeStamp: '. $p_obj->getTimeStamp(), __FILE__, __LINE__, __METHOD__,10);
							$punch_pairs[$p_obj->getPunchControlID()][] = $p_obj->getStatus();
						}

						if ( isset($punch_pairs) ) {
							foreach($punch_pairs as $punch_control_id => $punch_pair) {
								if ( count($punch_pair) != 2 ) {
									Debug::text('Found Missing Punch: ', __FILE__, __LINE__, __METHOD__,10);

									if ( $punch_pair[0] == 10 ) { //Missing Out Punch
										Debug::text('Found Missing Out Punch: ', __FILE__, __LINE__, __METHOD__,10);

										$ef = new ExceptionFactory();
										$ef->setUserDateID( $user_date_id );
										$ef->setExceptionPolicyID( $ep_obj->getId() );
										$ef->setPunchControlID( $p_obj->getPunchControlID() );
										$ef->setType( 10 );
										$ef->setEnableDemerits( TRUE );
										if ( $ef->isValid() ) {
											$ef->Save();
										}

									}
								} else {
									Debug::text('No Missing Punches...', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						}
						unset($punch_pairs, $punch_pair);

						break;
					case 'p': //Missing Lunch In/Out punch
						foreach ( $plf as $p_obj ) {
							if ( $p_obj->getType() == 20 ) {
								Debug::text(' Punch: Status: '. $p_obj->getStatus() .' Punch Control ID: '. $p_obj->getPunchControlID() .' TimeStamp: '. $p_obj->getTimeStamp(), __FILE__, __LINE__, __METHOD__,10);
								$punch_pairs[] = $p_obj->getStatus();
							}
						}

						if ( isset($punch_pairs) AND ( count($punch_pairs) % 2 ) != 0 ) {
							Debug::text('Found Missing Lunch In/Out Punch: ', __FILE__, __LINE__, __METHOD__,10);

							$ef = new ExceptionFactory();
							$ef->setUserDateID( $user_date_id );
							$ef->setExceptionPolicyID( $ep_obj->getId() );
							$ef->setPunchControlID( $p_obj->getPunchControlID() );
							$ef->setType( 10 );
							$ef->setEnableDemerits( TRUE );
							if ( $ef->isValid() ) {
								$ef->Save();
							}
						} else {
							Debug::text('Lunch Punches match up.', __FILE__, __LINE__, __METHOD__,10);
						}
						unset($punch_pairs);

						break;
					case 'g': //Over Scheduled Hours
						//This ONLY takes in to account WORKED hours, not paid absence hours.

						$schedule_total_time = 0;

						if ( $slf->getRecordCount() > 0 ) {
							//Check for schedule policy
							foreach ( $slf as $s_obj ) {
								Debug::text(' Schedule Total Time: '. $s_obj->getTotalTime(), __FILE__, __LINE__, __METHOD__,10);

								$schedule_total_time += $s_obj->getTotalTime();
							}

							$daily_total_time=0;
							if ( $schedule_total_time > 0 ) {
								//Get daily total time.
								$udtlf = new UserDateTotalListFactory();
								$udtlf->getByUserDateIdAndStatus( $user_date_id, 20 );
								if ( $udtlf->getRecordCount() > 0 ) {
									foreach( $udtlf as $udt_obj ) {
										$daily_total_time += $udt_obj->getTotalTime();
									}
								}

								Debug::text(' Daily Total Time: '. $daily_total_time .' Schedule Total Time: '. $schedule_total_time, __FILE__, __LINE__, __METHOD__,10);

								if ( $daily_total_time > 0 AND $daily_total_time > $schedule_total_time ) {
									Debug::text(' Worked Over Scheduled Hours', __FILE__, __LINE__, __METHOD__,10);

									$ef = new ExceptionFactory();
									$ef->setUserDateID( $user_date_id );
									$ef->setExceptionPolicyID( $ep_obj->getId() );
									$ef->setType( 10 );
									$ef->setEnableDemerits( TRUE );
									if ( $ef->isValid() ) {
										$ef->Save();
									}

								} else {
									Debug::text(' DID NOT Work Over Scheduled Hours', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						} else {
							Debug::text(' Not Scheduled', __FILE__, __LINE__, __METHOD__,10);
						}

						break;
					case 'h': //Under Scheduled Hours
						//This ONLY takes in to account WORKED hours, not paid absence hours.

						$schedule_total_time = 0;

						if ( $slf->getRecordCount() > 0 ) {
							//Check for schedule policy
							foreach ( $slf as $s_obj ) {
								Debug::text(' Schedule Total Time: '. $s_obj->getTotalTime(), __FILE__, __LINE__, __METHOD__,10);

								$schedule_total_time += $s_obj->getTotalTime();
							}

							$daily_total_time=0;
							if ( $schedule_total_time > 0 ) {
								//Get daily total time.
								$udtlf = new UserDateTotalListFactory();
								$udtlf->getByUserDateIdAndStatus( $user_date_id, 20 );
								if ( $udtlf->getRecordCount() > 0 ) {
									foreach( $udtlf as $udt_obj ) {
										$daily_total_time += $udt_obj->getTotalTime();
									}
								}

								Debug::text(' Daily Total Time: '. $daily_total_time .' Schedule Total Time: '. $schedule_total_time, __FILE__, __LINE__, __METHOD__,10);

								if ( $daily_total_time < $schedule_total_time ) {
									Debug::text(' Worked Under Scheduled Hours', __FILE__, __LINE__, __METHOD__,10);

									$ef = new ExceptionFactory();
									$ef->setUserDateID( $user_date_id );
									$ef->setExceptionPolicyID( $ep_obj->getId() );
									$ef->setType( 10 );
									$ef->setEnableDemerits( TRUE );
									if ( $ef->isValid() ) {
										$ef->Save();
									}

								} else {
									Debug::text(' DID NOT Work Under Scheduled Hours', __FILE__, __LINE__, __METHOD__,10);
								}
							}
						} else {
							Debug::text(' Not Scheduled', __FILE__, __LINE__, __METHOD__,10);
						}

						break;
					case 'b': //Not Scheduled
						$schedule_total_time = 0;

						if ( $slf->getRecordCount() == 0 ) {

							if ( $plf->getRecordCount() > 0 ) {
								Debug::text(' Worked when wasnt scheduled', __FILE__, __LINE__, __METHOD__,10);

								$ef = new ExceptionFactory();
								$ef->setUserDateID( $user_date_id );
								$ef->setExceptionPolicyID( $ep_obj->getId() );
								$ef->setType( 10 );
								$ef->setEnableDemerits( TRUE );
								if ( $ef->isValid() ) {
									$ef->Save();
								}

							}
						} else {
							Debug::text(' IS Scheduled', __FILE__, __LINE__, __METHOD__,10);
						}

						break;
					case 'o': //No Lunch
						//If they are scheduled or not, we can check for a meal policy and base our
						//decision off that. We don't want a No Lunch exception on a 3hr shift though.
						//Also ignore this exception if the lunch is auto-deduct.

						$daily_total_time = 0;

						$udtlf = new UserDateTotalListFactory();
						$udtlf->getByUserDateIdAndStatus( $user_date_id, 20 );
						if ( $udtlf->getRecordCount() > 0 ) {
							foreach( $udtlf as $udt_obj ) {
								$daily_total_time += $udt_obj->getTotalTime();
							}
						}
						Debug::text('Day Total Time: '. $daily_total_time, __FILE__, __LINE__, __METHOD__,10);

						if ( $daily_total_time > 0 ) {
							//Check for lunch punch.
							$lunch_punch = FALSE;
							foreach ( $plf as $p_obj ) {
								if ( $p_obj->getType() == 20 ) {
									Debug::text('Found Lunch Punch: '. $p_obj->getTimeStamp(), __FILE__, __LINE__, __METHOD__,10);
									$lunch_punch = TRUE;
									break;
								}
							}

							if ( $lunch_punch == FALSE ) {
								Debug::text('DID NOT Find Lunch Punch... Checking meal policies. ', __FILE__, __LINE__, __METHOD__,10);

								//Use scheduled meal policy first.
								if ( $slf->getRecordCount() > 0 ) {
									Debug::text('Schedule Found...', __FILE__, __LINE__, __METHOD__,10);
									foreach ( $slf as $s_obj ) {
										if ( $s_obj->getSchedulePolicyObject() !== FALSE
												AND $s_obj->getSchedulePolicyObject()->getMealPolicyObject() !== FALSE
												AND $s_obj->getSchedulePolicyObject()->getMealPolicyObject()->getType() != 10 ) {
											Debug::text('Found Schedule Meal Policy... Trigger Time: '. $s_obj->getSchedulePolicyObject()->getMealPolicyObject()->getTriggerTime(), __FILE__, __LINE__, __METHOD__,10);
											if ( $daily_total_time > $s_obj->getSchedulePolicyObject()->getMealPolicyObject()->getTriggerTime() ) {
												Debug::text('Daily Total Time is After Schedule Meal Policy Trigger Time: ', __FILE__, __LINE__, __METHOD__,10);

												$ef = new ExceptionFactory();
												$ef->setUserDateID( $user_date_id );
												$ef->setExceptionPolicyID( $ep_obj->getId() );
												$ef->setType( 10 );
												$ef->setEnableDemerits( TRUE );
												if ( $ef->isValid() ) {
													$ef->Save();
												}
											}

										} else {
											Debug::text('Schedule Meal Policy does not exist, or is auto-deduct?', __FILE__, __LINE__, __METHOD__,10);
										}
									}
								} else {
									Debug::text('No Schedule Found...', __FILE__, __LINE__, __METHOD__,10);

									//Check if they have a meal policy, with no schedule.
									$mplf = new MealPolicyListFactory();
									$mplf->getByPolicyGroupUserId( $user_date_obj->getUser() );
									if ( $mplf->getRecordCount() > 0 ) {
										Debug::text('Found UnScheduled Meal Policy...', __FILE__, __LINE__, __METHOD__,10);

										$m_obj = $mplf->getCurrent();
										if ( $daily_total_time > $m_obj->getTriggerTime()
												AND $m_obj->getType() != 10 ) {
											Debug::text('Daily Total Time is After Schedule Meal Policy Trigger Time: '. $m_obj->getTriggerTime(), __FILE__, __LINE__, __METHOD__,10);

											$ef = new ExceptionFactory();
											$ef->setUserDateID( $user_date_id );
											$ef->setExceptionPolicyID( $ep_obj->getId() );
											$ef->setType( 10 );
											$ef->setEnableDemerits( TRUE );
											if ( $ef->isValid() ) {
												$ef->Save();
											}
										} else {
											Debug::text('Auto-deduct meal policy, ignorning this exception.', __FILE__, __LINE__, __METHOD__,10);
										}
									}
								}

							} else {
								Debug::text('Found Lunch Punch... Ignoring this exception. ', __FILE__, __LINE__, __METHOD__,10);
							}

						}


						break;

					default:
						Debug::text('BAD, should never get here: ', __FILE__, __LINE__, __METHOD__,10);
						break;

				}


			}
		}

		$profiler->stopTimer( "ExceptionPolicy::calcExceptions()");

		return TRUE;
	}


	function Validate() {
		/*
		if ( $this->getDeleted() == TRUE ){
			//Check to make sure there are no hours using this OT policy.
			$udtlf = new UserDateTotalListFactory();
			$udtlf->getByAbsencePolicyId( $this->getId() );
			if ( $udtlf->getRecordCount() > 0 ) {
				$this->Validator->isTRUE(	$this->inuse_validator_label,
											FALSE,
											$this->inuse_validator_msg);

			}
		}
		*/
		return TRUE;
	}

	function preSave() {
		return TRUE;
	}

	function postSave() {
		return TRUE;
	}
}
?>
