<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 556 $
 * $Id: RequestFactory.class.php 556 2007-01-22 18:52:01Z ipso $
 * $Date: 2007-01-22 10:52:01 -0800 (Mon, 22 Jan 2007) $
 */
class RequestFactory extends Factory {
	protected $table = 'request';
	protected $pk_sequence_name = 'request_id_seq'; //PK Sequence name

	var $user_date_obj = NULL;

	protected $type_options = array(
										10 => 'Missed Punch',
										20 => 'Time Adjustment',
										30 => 'Absence (incl. Vacation)',
										40 => 'Schedule Adjustment',
										100 => 'Other',
									);

	protected $status_options = array(
										10 => 'INCOMPLETE',
										20 => 'OPEN',
										30 => 'PENDING AUTHORIZATION',
										40 => 'AUTHORIZATION OPEN',
										50 => 'ACTIVE',
										55 => 'AUTHORIZATION DECLINED',
										60 => 'DISABLED'
									);



	protected $userdate_validator_msg = 'Invalid User Date ID';
	protected $userdate_validator_label = 'user_date';

	protected $status_validator_options_msg = 'Incorrect Status';
	protected $status_validator_label = 'status';

	protected $type_validator_options_msg = 'Incorrect Type';
	protected $type_validator_label = 'type';

	protected $message_validator_length_msg = 'Invalid message length';
	protected $message_validator_label = 'message';

	function getUserDateObject() {
		if ( is_object($this->user_date_obj) ) {
			return $this->user_date_obj;
		} else {
			$udlf = new UserDateListFactory();
			$this->user_date_obj = $udlf->getById( $this->getUserDateID() )->getCurrent();

			return $this->user_date_obj;
		}
	}

	//Used for authorizationFactory
	function getUserObject() {
		return $this->getUserDateObject()->getUserObject();
	}

	//Used for authorizationFactory
	function getUser() {

		return $this->getUserDateObject()->getUser();
	}

	function setUserDate($user_id, $date) {
		$user_date_id = UserDateFactory::findOrInsertUserDate( $user_id, $date);
		Debug::text(' User Date ID: '. $user_date_id, __FILE__, __LINE__, __METHOD__,10);
		if ( $user_date_id != '' ) {
			$this->setUserDateID( $user_date_id );
			return TRUE;
		}
		Debug::text(' No User Date ID found', __FILE__, __LINE__, __METHOD__,10);

		return FALSE;
	}

	function getUserDateID() {
		if ( isset($this->data['user_date_id']) ) {
			return $this->data['user_date_id'];
		}

		return FALSE;
	}
	function setUserDateID($id = NULL) {
		$id = trim($id);

		$udlf = new UserDateListFactory();

		if (  $this->Validator->isResultSetWithRows(	$this->userdate_validator_label,
														$udlf->getByID($id),
														$this->userdate_validator_msg
														) ) {
			$this->data['user_date_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getType() {
		if ( isset($this->data['type_id']) ) {
			return $this->data['type_id'];
		}

		return FALSE;
	}
	function setType($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('type') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->type_validator_label,
											$value,
											$this->type_validator_options_msg,
											$this->getOptions('type')) ) {

			$this->data['type_id'] = $value;

			return FALSE;
		}

		return FALSE;
	}

	function getStatus() {
		if ( isset($this->data['status_id']) ) {
			return $this->data['status_id'];
		}

		return FALSE;
	}
	function setStatus($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('status') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->status_validator_label,
											$value,
											$this->status_validator_options_msg,
											$this->getOptions('status')) ) {

			$this->data['status_id'] = $value;

			return FALSE;
		}

		return FALSE;
	}

	function getAuthorized() {
		if ( isset($this->data['authorized']) AND $this->data['authorized'] !== NULL) {
			return $this->fromBool( $this->data['authorized'] );
		}

		return NULL;
	}
	function setAuthorized($bool) {
		$this->data['authorized'] = $this->toBool($bool);

		return true;
	}

	function getMessage() {
		if ( isset($this->tmp_data['message']) ) {
			return $this->tmp_data['message'];
		}

		return FALSE;
	}
	function setMessage($text) {
		$text = trim($text);

		if 	(	$this->Validator->isLength(		$this->message_validator_label,
												$text,
												$this->message_validator_length_msg,
												5,
												1024) ) {

			$this->tmp_data['message'] = htmlentities( $text );

			return TRUE;
		}

		return FALSE;
	}

	function Validate() {
		if (	$this->isNew() == TRUE
				AND $this->Validator->hasError($this->message_validator_label) == FALSE
				AND $this->getMessage() == FALSE ) {
			$this->Validator->isTRUE(		$this->message_validator_label,
											FALSE,
											$this->message_validator_length_msg );
		}

		if ( $this->getUserDateID() == FALSE ) {
			$this->Validator->isTRUE(		$this->userdate_validator_label,
											FALSE,
											$this->userdate_validator_msg );
		}

		return TRUE;
	}

	function preSave() {
		unset($this->data['date_stamp']);

		return TRUE;
	}

	function postSave() {
		//Save message here after we have the request_id.
		//if ( $this->isNew() == TRUE ) {
		if ( $this->getMessage() !== FALSE ) {
			$mf = new MessageFactory();
			$mf->setObjectType( 50 ); //Request
			$mf->setObject( $this->getID() );
			$mf->setParent( 0 );
			$mf->setPriority();
			$mf->setStatus('UNREAD');
			$mf->setBody( $this->getMessage() );

			return $mf->Save();
		}

		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Request - Type: '. $this->getType(), NULL, $this->getTable() );
	}
}
?>
